﻿/**
 * Paperskated3d - simple skateboard simulator in Papervision3d
 * 
 * @author Bartek Drozdz 
 * @version 1.0
 * 
 * Released under Creative Commons Attribution-Non-Commercial-Share Alike 3.0 License.
 */
package com.paperskate3d {
	
	import com.paperskate3d.gui.HelpAbout;
	import com.paperskate3d.gui.HiLow;
	import com.paperskate3d.gui.Instruction;
	import com.paperskate3d.gui.LowTextField;
	import com.paperskate3d.gui.SpeedMeter;
	import com.paperskate3d.gui.WelcomeScreen;
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.display.BlendMode;
	import flash.events.Event;
	import com.paperskate3d.gui.PSUIEvent;
	import net.hires.utils.Stats;

	import flash.display.StageQuality;
	import flash.display.StageScaleMode;
	import flash.display.StageAlign;
	
	import caurina.transitions.Tweener;
	import caurina.transitions.properties.ColorShortcuts;
	
	import com.paperskate3d.gui.SpeedState;
	
	import com.carlcalderon.arthropod.Debug;
	
	/* NOTE: works only with GreatWhite rev. 638 or older */
	public class SkateDocument extends Sprite {
	
		[Embed(source = '../../../assets/paperskate-logo.jpg')]
		private var PSLogo:Class;
		private var logo:Bitmap;
		
		private var speedState:SpeedState;
		private var welcome:WelcomeScreen;
		private var instruction:Instruction;
		private var hilo:HiLow;
		private var helab:HelpAbout;
		private var speedMeter:SpeedMeter;
		private var game:SkateView;
		
		private var stats:Stats;
		
		private var initActivation:Boolean = true;
		
		public function SkateDocument() {
			stage.quality = StageQuality.LOW;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			
			loaderInfo.addEventListener(Event.COMPLETE, init);
		}
		
		private function init(e:Event):void {
			ColorShortcuts.init();
			
			speedMeter = new SpeedMeter();
			game = new SkateView(speedMeter);
			welcome = new WelcomeScreen();
			instruction = new Instruction();
			logo = new PSLogo() as Bitmap;
			logo.blendMode = BlendMode.SCREEN;
			speedState = new SpeedState();
			hilo = new HiLow();
			helab = new HelpAbout();
			stats = new Stats();
			
			addChild(logo);
			addChild(hilo);
			addChild(helab);
			
			addChild(welcome);

			game.addEventListener(SkatingEvent.SPEED_MODE_CHANGE, adjustSpeedState);
			game.addEventListener(SkatingEvent.TOGGLE_STATS, toggleStats);
			
			welcome.addEventListener(PSUIEvent.WELCOME_SCREEN_CLEARED, continueAfterWelcome);
			game.addEventListener(PSUIEvent.GAME_CLEARED, launchInstruction);
			instruction.addEventListener(PSUIEvent.INSTRUCTION_CLEARED, backToGame);
			hilo.addEventListener(PSUIEvent.QUALITY_CHANGE, onQualityChange);
			helab.addEventListener(PSUIEvent.OPEN_INSTRUCTION, openInstruction);
			
			resize();
			stage.addEventListener(Event.RESIZE, resize);
			
			addEventListener(Event.ACTIVATE, activate);
			addEventListener(Event.DEACTIVATE, deactivate);
		}
		
		private function toggleStats(ske:SkatingEvent):void {
			if (contains(stats)) removeChild(stats);
			else addChild(stats);
		}

		private function continueAfterWelcome(psuie:PSUIEvent):void {
			removeChild(welcome);
			if (psuie.startGame) {
				addChild(game);
				addChild(speedMeter);
				hilo.activate();
			} else {
				addChild(instruction);
			}
		}
		
		private function openInstruction(psuie:PSUIEvent):void {
			Tweener.setTimeScale(1);
			if (contains(welcome)) {
				welcome.forceClear();
			} else if (contains(game)) {
				game.exitGame();
			}
		}
		
		private function onQualityChange(psuie:PSUIEvent):void {
			game.setRenderingQuality(psuie.isHigh);
		}
		
		private function backToGame(psuie:PSUIEvent):void {
			removeChild(instruction);
			addChild(game);
			addChild(speedMeter);
			hilo.activate();
		}

		private function launchInstruction(psuie:PSUIEvent):void {
			Tweener.setTimeScale(1);
			removeChild(game);
			removeChild(speedMeter);
			addChild(instruction);
			hilo.deactivate();
		}
		
		private function activate(e:Event):void {
			if (initActivation) {
				initActivation = false;
				return;
			}
			
			if(!game.isPaused()) Tweener.resumeAllTweens();
			game.startRendering();
		}
		
		private function deactivate(e:Event):void {
			if(!game.isPaused()) Tweener.pauseAllTweens();
			game.stopRendering();
		}

		private function adjustSpeedState(ske:SkatingEvent):void {
			speedState.setState(ske);
			switch(ske.speedMode) {
				case SkatingEvent.SPEED_NORMAL:
					if (contains(speedState)) removeChild(speedState);
					break;
				case SkatingEvent.SPEED_SLOW:
					if (!contains(speedState)) addChild(speedState);
					break;
				case SkatingEvent.SPEED_PAUSE:
					if (!contains(speedState)) addChild(speedState);
					break;
			}
		}

		private function resize(e:Event = null):void {
			var mw:Number = Math.round(stage.stageWidth / 2);
			var mh:Number = Math.round(stage.stageHeight / 2);
			
			speedState.x = mw;
			speedState.y = mh + 235;
			
			welcome.x = mw;
			welcome.y = mh;
			
			game.x = Math.round(mw - SkateView.GAME_WIDTH / 2);
			game.y = Math.round(mh - SkateView.GAME_HEIGHT / 2);
			
			speedMeter.x = Math.round(mw - SpeedMeter.METER_SIZE / 2);
			speedMeter.y = 32;
			
			hilo.x = stage.stageWidth - (10 + hilo.width);
			hilo.y = stage.stageHeight - (10 + hilo.height);
			
			helab.x = stage.stageWidth - (10 + helab.width);
			helab.y = 10;
			
			stats.x = 0;
			stats.y = stage.stageHeight - 75;
			
			instruction.x = mw;
			instruction.y = mh;
		}
	}
}








