﻿/**
 * Paperskated3d - simple skateboard simulator in Papervision3d
 * 
 * @author Bartek Drozdz 
 * @version 1.0
 * 
 * Released under Creative Commons Attribution-Non-Commercial-Share Alike 3.0 License.
 */
package com.paperskate3d {
	
	import com.paperskate3d.gui.PSUIEvent;
	import com.paperskate3d.terrain.ITerrain;
	import com.paperskate3d.terrain.TerrainFactory;
	import com.paperskate3d.tricks.TrickManager;
	import com.paperskate3d.tricks.BasicTrick;
	import com.paperskate3d.tricks.KickFlip;
	import com.paperskate3d.tricks.Ollie;
	import com.paperskate3d.tricks.Spin;
	import flash.filters.BlurFilter;
	import flash.filters.ConvolutionFilter;
	import flash.filters.GlowFilter;
	import flash.geom.ColorTransform;
	import flash.utils.Timer;
	import flash.events.TimerEvent;
	import org.papervision3d.lights.PointLight3D;
	import org.papervision3d.objects.DisplayObject3D;

	import flash.events.KeyboardEvent;
	import flash.ui.Keyboard;
	import flash.events.MouseEvent;
	import flash.display.BlendMode;
	import flash.display.StageQuality;
	
	import flash.events.Event;
	import org.papervision3d.view.BasicView;
	import com.paperskate3d.skate.SkateBoard;
	import com.paperskate3d.gui.SpeedMeter;
	
	import caurina.transitions.Tweener;
	import com.paperskate3d.util.SkateTransition;
	
	import com.carlcalderon.arthropod.Debug;

	public class SkateView extends BasicView {
		
		private var container:DisplayObject3D;
		private var skate:SkateBoard;
		private var terrain:ITerrain;
		private var light:PointLight3D;
		private var trickManager:TrickManager;
		private var operator:Operator;

		private var speed:Number = 0;
		
		private var speedMeter:SpeedMeter;
		
		private var groundFriction:Number = 1;// 0.999;
		private var trickSlowdown:Number = 0.99;
		
		public static var MAX_SPEED:Number = 100;
		public static var MINIMAL_TRICK_SPEED:Number = 20;
		
		private var pause:Boolean = false;

		public var playbackSpeed:Number = 1;
		
		public static var GAME_WIDTH:Number = 750;
		public static var GAME_HEIGHT:Number = 900;

		public function SkateView(sm:SpeedMeter) {
			super(GAME_WIDTH, GAME_HEIGHT, false, false);
			
			focusRect = false;
			
			container = new DisplayObject3D();
			
			skate = new SkateBoard();
			container.addChild(skate);
			
			terrain = TerrainFactory.getTerrain();
			container.addChild(TerrainFactory.castAsDo3D(terrain));

			scene.addChild(container);
			container.y = -150;
			
			camera.lookAt(skate);
			operator = new Operator(camera);
			
			speedMeter = sm;
			addEventListener(SkatingEvent.SPEED_CHANGE, speedMeter.onSpeedChange);
			
			trickManager = new TrickManager(skate);
			trickManager.addEventListener(SkatingEvent.TRICK_STARTED, onTrickStarted);
			trickManager.addEventListener(SkatingEvent.TRICK_FINISHED, onTrickFinished);
			
			addEventListener(Event.ADDED_TO_STAGE, init);
		}
		
		public function init(e:Event):void {
			/** Weird, but otherwise the elements are not positioned correctly */
			singleRender();
			singleRender();
			SkateTransition.makeIn(this, afterInit);
		}
		
		private function afterInit():void {
			stage.addEventListener(KeyboardEvent.KEY_UP, keyUp);
			stage.addEventListener(KeyboardEvent.KEY_DOWN, keyDown);
			stage.addEventListener(MouseEvent.MOUSE_WHEEL, onMouseWheel);
			stage.focus = this;
			startRendering();
		}
		
		public function isPaused():Boolean {
			return pause;
		}
		
		private function onMouseWheel(me:MouseEvent):void {
			speedUp();
		}
		
		private function speedUp():void {
			if (!trickManager.trickInProgress()) {
				speed += 2;
				speed = Math.min(MAX_SPEED, speed);
				
				var ske:SkatingEvent = new SkatingEvent(SkatingEvent.SPEED_CHANGE);
				ske.speed = speed;
				dispatchEvent(ske); 
			}
		}
		
		public function keyDown(ke:KeyboardEvent):void {
			if (ke.keyCode == 80 || ke.keyCode == 90) {
				checkSpeedmodeChange(ke);
				return;
			}
			
			if (ke.keyCode == 72) { // H
				exitGame();
				return;
			}
			
			if (ke.keyCode == 76) { // L
				speedUp();
				return;
			}
			
			if (ke.keyCode == 86) { // V
				dispatchEvent(new SkatingEvent(SkatingEvent.TOGGLE_STATS));
				return;
			}

			operator.keyDown(ke);
			
			if (speed > MINIMAL_TRICK_SPEED && !pause) {
				var trickSuccess:Boolean = trickManager.perform(ke, speed, playbackSpeed);
				
				if (trickSuccess) {
					speed *= trickSlowdown;
					var ske:SkatingEvent = new SkatingEvent(SkatingEvent.SPEED_CHANGE);
					ske.speed = speed;
					dispatchEvent(ske); 
				}
			}
		}
		
		public function setRenderingQuality(hi:Boolean):void {
			skate.hiquality = hi;
			if (hi) stage.quality = StageQuality.MEDIUM;
			else stage.quality = StageQuality.LOW;
		}
		
		public function checkSpeedmodeChange(ke:KeyboardEvent):void {
			
			var sk:SkatingEvent = new SkatingEvent(SkatingEvent.SPEED_MODE_CHANGE);

			if (ke.keyCode == 80) { // P for Pause
				if (pause) {
					Tweener.resumeAllTweens();
					pause = false;
					if (playbackSpeed == .1) {
						sk.speedMode = SkatingEvent.SPEED_SLOW;
					} else {
						sk.speedMode = SkatingEvent.SPEED_NORMAL;
					}
				} else {
					Tweener.pauseAllTweens();
					pause = true;
					sk.speedMode = SkatingEvent.SPEED_PAUSE;
				}
				dispatchEvent(sk);
			} else if (ke.keyCode == 90 && !pause) { // Z for Slow Motion
				playbackSpeed = (playbackSpeed == 1) ? .1 : 1;
				if (playbackSpeed == .1) {
					sk.speedMode = SkatingEvent.SPEED_SLOW;
				} else {
					sk.speedMode = SkatingEvent.SPEED_NORMAL;
				}
				dispatchEvent(sk);
				Tweener.setTimeScale(playbackSpeed);
			}
			
			
		}
		
		public function keyUp(ke:KeyboardEvent):void {
			operator.keyUp(ke);
		}
		
		public function exitGame():void {
			stopRendering();
			speed = 0;
			pause = false;
			playbackSpeed = 1;
			Tweener.removeAllTweens();
			trickManager.abort();
			skate.reset();
			
			stage.removeEventListener(KeyboardEvent.KEY_UP, keyUp);
			stage.removeEventListener(KeyboardEvent.KEY_DOWN, keyDown);
			stage.removeEventListener(MouseEvent.MOUSE_WHEEL, speedUp);

			SkateTransition.makeOut(this, onGameCleared);
			
			var sk:SkatingEvent = new SkatingEvent(SkatingEvent.SPEED_MODE_CHANGE);
			sk.speedMode = SkatingEvent.SPEED_NORMAL;
			dispatchEvent(sk);
			
			speedMeter.clear();
		}
		
		private function onGameCleared():void {
			dispatchEvent(new PSUIEvent(PSUIEvent.GAME_CLEARED));
		}
		
		override protected function onRenderTick(event:Event = null):void {
			
			var realSpeed:Number = (pause) ? 0 : speed * playbackSpeed;	
			terrain.move(realSpeed);
			super.onRenderTick(event);
			
			operator.tick();
			
			
			skate.move(realSpeed * 3, trickManager.getWheelSituation());
			
			if (!pause && playbackSpeed == 1) {
				speed *= groundFriction;
				
				var ske:SkatingEvent = new SkatingEvent(SkatingEvent.SPEED_CHANGE);
				ske.speed = speed;
				dispatchEvent(ske); 
			}
			
			if (speed < MINIMAL_TRICK_SPEED) trickManager.lowSpeed();
			
		}
		
		public function onTrickStarted(e:Event):void {
			addEventListener(Event.ENTER_FRAME, updateShadow);
		}
		
		public function updateShadow(e:Event):void {
			terrain.updateShadow(skate);
		}
		
		public function onTrickFinished(e:Event):void {
			removeEventListener(Event.ENTER_FRAME, updateShadow);
			terrain.resetShadow();
		}
	}
}